@@ -26,13 +26,18 @@
 #include <unistd.h>
 #include <inttypes.h>
 
-#include <osmocom/sigtran/osmo_ss7.h>
-
 #include <osmocom/core/linuxlist.h>
 #include <osmocom/core/utils.h>
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/logging.h>
 
+#include <osmocom/sigtran/osmo_ss7.h>
+#include <osmocom/sigtran/protocol/m3ua.h>
+
+#include "ss7_as.h"
+#include "ss7_asp.h"
+#include "ss7_route.h"
+#include "ss7_route_table.h"
 #include "ss7_internal.h"
 #include "xua_as_fsm.h"
 #include "xua_asp_fsm.h"
@@ -44,7 +49,7 @@
 struct value_string osmo_ss7_as_traffic_mode_vals[] = {
 	{ OSMO_SS7_AS_TMOD_BCAST,	"broadcast" },
 	{ OSMO_SS7_AS_TMOD_LOADSHARE,	"loadshare" },
-	{ OSMO_SS7_AS_TMOD_ROUNDROBIN,	"round-robin" },
+	{ OSMO_SS7_AS_TMOD_ROUNDROBIN,	"roundrobin" },
 	{ OSMO_SS7_AS_TMOD_OVERRIDE,	"override" },
 	{ 0, NULL }
 };
@@ -93,25 +98,28 @@ struct osmo_ss7_as *ss7_as_alloc(struct osmo_ss7_instance *inst, const char *nam
 	return as;
 }
 
+/*! \brief Get asp_protocol configuration of a given AS
+ *  \param[in] as Application Server in which to look for \ref asp_protocol
+ *  \returns The asp_protocol this AS is configured with */
+enum osmo_ss7_asp_protocol osmo_ss7_as_get_asp_protocol(const struct osmo_ss7_as *as)
+{
+	return as->cfg.proto;
+}
+
 /*! \brief Add given ASP to given AS
  *  \param[in] as Application Server to which \ref asp is added
  *  \param[in] asp Application Server Process to be added to \ref as
  *  \returns 0 on success; negative in case of error */
-int osmo_ss7_as_add_asp(struct osmo_ss7_as *as, const char *asp_name)
+int ss7_as_add_asp(struct osmo_ss7_as *as, struct osmo_ss7_asp *asp)
 {
-	struct osmo_ss7_asp *asp;
 	unsigned int i;
-
-	OSMO_ASSERT(ss7_initialized);
-	asp = osmo_ss7_asp_find_by_name(as->inst, asp_name);
-	if (!asp)
-		return -ENODEV;
-
-	LOGPAS(as, DLSS7, LOGL_INFO, "Adding ASP %s to AS\n", asp->cfg.name);
+	OSMO_ASSERT(asp);
 
 	if (osmo_ss7_as_has_asp(as, asp))
 		return 0;
 
+	LOGPAS(as, DLSS7, LOGL_INFO, "Adding ASP %s to AS\n", asp->cfg.name);
+
 	for (i = 0; i < ARRAY_SIZE(as->cfg.asps); i++) {
 		if (!as->cfg.asps[i]) {
 			as->cfg.asps[i] = asp;
@@ -124,6 +132,22 @@ int osmo_ss7_as_add_asp(struct osmo_ss7_as *as, const char *asp_name)
 	return -ENOSPC;
 }
 
+/*! \brief Add given ASP to given AS
+ *  \param[in] as Application Server to which \ref asp is added
+ *  \param[in] asp_name Name of Application Server Process to be added to \ref as
+ *  \returns 0 on success; negative in case of error */
+int osmo_ss7_as_add_asp(struct osmo_ss7_as *as, const char *asp_name)
+{
+	struct osmo_ss7_asp *asp;
+
+	OSMO_ASSERT(ss7_initialized);
+	asp = osmo_ss7_asp_find_by_name(as->inst, asp_name);
+	if (!asp)
+		return -ENODEV;
+
+	return ss7_as_add_asp(as, asp);
+}
+
 /*! \brief Delete given ASP from given AS
  *  \param[in] as Application Server from which \ref asp is deleted
  *  \param[in] asp Application Server Process to delete from \ref as
@@ -154,8 +178,6 @@ int osmo_ss7_as_del_asp(struct osmo_ss7_as *as, const char *asp_name)
  *  \param[in] as Application Server to destroy */
 void osmo_ss7_as_destroy(struct osmo_ss7_as *as)
 {
-	struct osmo_ss7_route *rt, *rt2;
-
 	OSMO_ASSERT(ss7_initialized);
 	LOGPAS(as, DLSS7, LOGL_INFO, "Destroying AS\n");
 
@@ -163,10 +185,7 @@ void osmo_ss7_as_destroy(struct osmo_ss7_as *as)
 		osmo_fsm_inst_term(as->fi, OSMO_FSM_TERM_REQUEST, NULL);
 
 	/* find any routes pointing to this AS and remove them */
-	llist_for_each_entry_safe(rt, rt2, &as->inst->rtable_system->routes, list) {
-		if (rt->dest.as == as)
-			osmo_ss7_route_destroy(rt);
-	}
+	ss7_route_table_del_routes_by_as(as->inst->rtable_system, as);
 
 	as->inst = NULL;
 	llist_del(&as->list);
@@ -191,6 +210,21 @@ bool osmo_ss7_as_has_asp(const struct osmo_ss7_as *as,
 	return false;
 }
 
+/*! Determine amount of ASPs associated to an AS.
+ *  \param[in] as Application Server.
+ *  \returns number of ASPs associated to as */
+unsigned int osmo_ss7_as_count_asp(const struct osmo_ss7_as *as)
+{
+	unsigned int i;
+	unsigned int cnt = 0;
+
+	for (i = 0; i < ARRAY_SIZE(as->cfg.asps); i++) {
+		if (as->cfg.asps[i])
+			cnt++;
+	}
+	return cnt;
+}
+
 /*! Determine if given AS is in the active state.
  *  \param[in] as Application Server.
  *  \returns true in case as is active; false otherwise. */
@@ -212,3 +246,114 @@ bool osmo_ss7_as_down(const struct osmo_ss7_as *as)
 		return true;
 	return as->fi->state == XUA_AS_S_DOWN;
 }
+
+static struct osmo_ss7_asp *ss7_as_select_asp_override(struct osmo_ss7_as *as)
+{
+	struct osmo_ss7_asp *asp;
+	unsigned int i;
+
+	/* FIXME: proper selection of the ASP based on the SLS! */
+	for (i = 0; i < ARRAY_SIZE(as->cfg.asps); i++) {
+		asp = as->cfg.asps[i];
+		if (asp && osmo_ss7_asp_active(asp))
+			break;
+	}
+	return asp;
+}
+
+static struct osmo_ss7_asp *ss7_as_select_asp_roundrobin(struct osmo_ss7_as *as)
+{
+	struct osmo_ss7_asp *asp;
+	unsigned int i;
+	unsigned int first_idx;
+
+	first_idx = (as->cfg.last_asp_idx_sent + 1) % ARRAY_SIZE(as->cfg.asps);
+	i = first_idx;
+	do {
+		asp = as->cfg.asps[i];
+		if (asp && osmo_ss7_asp_active(asp))
+			break;
+		i = (i + 1) % ARRAY_SIZE(as->cfg.asps);
+	} while (i != first_idx);
+	as->cfg.last_asp_idx_sent = i;
+
+	return asp;
+}
+
+/* returns NULL if multiple ASPs would need to be selected. */
+static struct osmo_ss7_asp *ss7_as_select_asp_broadcast(struct osmo_ss7_as *as)
+{
+	struct osmo_ss7_asp *asp;
+	struct osmo_ss7_asp *asp_found = NULL;
+
+	for (unsigned int i = 0; i < ARRAY_SIZE(as->cfg.asps); i++) {
+		asp = as->cfg.asps[i];
+		if (!asp || !osmo_ss7_asp_active(asp))
+			continue;
+		if (asp_found) /* >1 ASPs selected, early return */
+			return NULL;
+		asp_found = asp;
+	}
+	return asp_found;
+}
+
+/*! Select an AS to transmit a message, according to AS configuration and ASP availability.
+ *  \param[in] as Application Server.
+ *  \returns asp to send the message to, NULL if no possible asp found
+ *
+ *  This function returns NULL too if multiple ASPs would be selected, ie. AS is
+ *  configured in broadcast mode and more than one ASP is configured.
+ */
+struct osmo_ss7_asp *osmo_ss7_as_select_asp(struct osmo_ss7_as *as)
+{
+	struct osmo_ss7_asp *asp = NULL;
+
+	switch (as->cfg.mode) {
+	case OSMO_SS7_AS_TMOD_OVERRIDE:
+		asp = ss7_as_select_asp_override(as);
+		break;
+	case OSMO_SS7_AS_TMOD_LOADSHARE:
+		/* TODO: actually use the SLS value to ensure same SLS goes
+		 * through same ASP. Not strictly required by M3UA RFC, but
+		 * would fit the overall principle. */
+	case OSMO_SS7_AS_TMOD_ROUNDROBIN:
+		asp = ss7_as_select_asp_roundrobin(as);
+		break;
+	case OSMO_SS7_AS_TMOD_BCAST:
+		return ss7_as_select_asp_broadcast(as);
+	case _NUM_OSMO_SS7_ASP_TMOD:
+		OSMO_ASSERT(false);
+	}
+
+	if (!asp) {
+		LOGPFSM(as->fi, "No selectable ASP in AS\n");
+		return NULL;
+	}
+	return asp;
+}
+
+bool osmo_ss7_as_tmode_compatible_xua(struct osmo_ss7_as *as, uint32_t m3ua_tmt)
+{
+	if (!as->cfg.mode_set_by_vty && !as->cfg.mode_set_by_peer)
+		return true;
+
+	switch (m3ua_tmt) {
+	case M3UA_TMOD_OVERRIDE:
+		if (as->cfg.mode == OSMO_SS7_AS_TMOD_OVERRIDE)
+			return true;
+		break;
+	case M3UA_TMOD_LOADSHARE:
+		if (as->cfg.mode == OSMO_SS7_AS_TMOD_LOADSHARE ||
+		    as->cfg.mode == OSMO_SS7_AS_TMOD_ROUNDROBIN)
+			return true;
+		break;
+	case M3UA_TMOD_BCAST:
+		if (as->cfg.mode == OSMO_SS7_AS_TMOD_BCAST)
+			return true;
+		break;
+	default:
+		break;
+	}
+	return false;
+
+}
