@@ -146,65 +146,6 @@ static int hmdt_message_for_distribution(struct osmo_ss7_instance *inst, struct
 	}
 }
 
-/*! Return human readable representation of the route, in a static buffer.
- * This uses both osmo_ss7_pointcode_print() and osmo_ss7_pointcode_print2(), so pairing
- * osmo_ss7_route_name() with osmo_ss7_pointcode_print() in the same printf statement is likely to
- * conflict.
- * \param[in] rt  The route information to print, or NULL.
- * \param[in] list_asps  If true, append info for all ASPs for the route's AS.
- * \returns A string constant or static buffer. */
-const char *osmo_ss7_route_name(struct osmo_ss7_route *rt, bool list_asps)
-{
-	static char buf[256];
-	char *pos = buf;
-	struct osmo_ss7_instance *inst;
-	size_t l;
-
-	if (!rt)
-		return "no route";
-
-	inst = rt->rtable->inst;
-
-#define APPEND(fmt, args ...) \
-	do { \
-		l = snprintf(pos, sizeof(buf) - (pos - buf), fmt, ## args); \
-		pos += l; \
-		if (pos - buf >= sizeof(buf) ) \
-			goto out; \
-	} while (0)
-
-	APPEND("pc=%u=%s mask=0x%x=%s",
-	       rt->cfg.pc, osmo_ss7_pointcode_print(inst, rt->cfg.pc),
-	       rt->cfg.mask, osmo_ss7_pointcode_print2(inst, rt->cfg.mask));
-
-	if (rt->dest.as) {
-		struct osmo_ss7_as *as = rt->dest.as;
-		int i;
-		APPEND(" via AS %s proto=%s", as->cfg.name, osmo_ss7_asp_protocol_name(as->cfg.proto));
-
-		if (list_asps) {
-			for (i = 0; i < ARRAY_SIZE(as->cfg.asps); i++) {
-				struct osmo_ss7_asp *asp = as->cfg.asps[i];
-				if (!asp)
-					continue;
-				APPEND(" ASP");
-				if (asp->cfg.name)
-					APPEND(" %s", asp->cfg.name);
-				if (asp->sock_name)
-					APPEND(" %s", asp->sock_name);
-			}
-		}
-	} else if (rt->dest.linkset)
-		APPEND(" via linkset %s", rt->dest.linkset->cfg.name);
-	else
-		APPEND(" has no route set");
-#undef APPEND
-
-out:
-	buf[sizeof(buf)-1] = '\0';
-	return buf;
-}
-
 /* HMDC->HMRT Msg For Routing; Figure 26/Q.704 */
 /* local message was receive d from L4, SRM, SLM, STM or SLTC, or
  * remote message received from L2 and HMDC determined msg for routing */
@@ -246,13 +187,12 @@ static int hmrt_message_for_routing(struct osmo_ss7_instance *inst,
 			case OSMO_SS7_ASP_PROT_M3UA:
 				DEBUGP(DLSS7, "rt->dest.as proto is M3UA for dpc=%u=%s\n",
 				       dpc, osmo_ss7_pointcode_print(inst, dpc));
-				return m3ua_tx_xua_as(as,xua);
+				return m3ua_tx_xua_as(as, xua);
 			case OSMO_SS7_ASP_PROT_IPA:
 				return ipa_tx_xua_as(as, xua);
 			default:
-				LOGP(DLSS7, LOGL_ERROR, "MTP message "
-					"for ASP of unknown protocol %u\n",
-					as->cfg.proto);
+				LOGP(DLSS7, LOGL_ERROR, "MTP message for ASP of unknown protocol %u\n",
+				     as->cfg.proto);
 				break;
 			}
 		} else if (rt->dest.linkset) {
@@ -262,15 +202,14 @@ static int hmrt_message_for_routing(struct osmo_ss7_instance *inst,
 				 * overwritten. */
 				const char *rt_name = osmo_ss7_route_name(rt, false);
 				LOGP(DLSS7, LOGL_ERROR,
-				     "Found route for dpc=%u=%s: %s,"
-				     " but MTP-TRANSFER.req unsupported for linkset.\n",
+				     "Found route for dpc=%u=%s: %s, but MTP-TRANSFER.req unsupported for linkset.\n",
 				     dpc, osmo_ss7_pointcode_print(inst, dpc), rt_name);
 			}
 		} else
 			OSMO_ASSERT(0);
 	} else {
-		LOGP(DLSS7, LOGL_ERROR, "MTP-TRANSFER.req for DPC %u: "
-			"no route!\n", dpc);
+		LOGP(DLSS7, LOGL_ERROR, "MTP-TRANSFER.req for dpc=%u=%s: no route!\n",
+		     dpc, osmo_ss7_pointcode_print(inst, dpc));
 		/* DPC unknown HMRT -> MGMT */
 		/* Message Received for inaccesible SP HMRT ->RTPC */
 		/* Discard Message */
