@@ -46,7 +46,12 @@
 #include "xua_internal.h"
 #include <osmocom/sigtran/sccp_sap.h>
 #include "sccp_internal.h"
+#include "ss7_as.h"
+#include "ss7_asp.h"
+#include "ss7_route.h"
+#include "ss7_route_table.h"
 #include "ss7_internal.h"
+#include "ss7_xua_srv.h"
 
 #include <netinet/tcp.h>
 
@@ -67,6 +72,20 @@
 	"SCTP (Stream Control Transmission Protocol)\n" \
 	"TCP (Transmission Control Protocol)\n"
 
+#define QOS_CLASS_RANGE_STR "<0-7>"
+#define QOS_CLASS_RANGE_HELP_STR "QoS Class\n"
+#define QOS_CLASS_VAR_STR "(" QOS_CLASS_RANGE_STR "|default)"
+#define QOS_CLASS_VAR_HELP_STR \
+	QOS_CLASS_RANGE_HELP_STR \
+	"Default QoS Class (0)\n"
+
+#define ROUTE_PRIO_RANGE_STR "<1-9>"
+#define ROUTE_PRIO_RANGE_HELP_STR "Priority\n"
+#define ROUTE_PRIO_VAR_STR "(" ROUTE_PRIO_RANGE_STR "|default)"
+#define ROUTE_PRIO_VAR_HELP_STR \
+	ROUTE_PRIO_RANGE_HELP_STR \
+	"Default Priority (5)\n"
+
 /* netinet/tcp.h */
 static const struct value_string tcp_info_state_values[] = {
 	{ TCP_ESTABLISHED,	"ESTABLISHED" },
@@ -234,7 +253,23 @@ DEFUN_ATTR(cs7_point_code, cs7_point_code_cmd,
 	return CMD_SUCCESS;
 }
 
-/* TODO: cs7 secondary-pc */
+DEFUN_ATTR(cs7_secondary_pc, cs7_secondary_pc_cmd,
+	   "secondary-pc POINT_CODE",
+	   "Configure the local Secondary Point Code\n"
+	   "Point Code\n",
+	   CMD_ATTR_IMMEDIATE)
+{
+	struct osmo_ss7_instance *inst = vty->index;
+	int pc = osmo_ss7_pointcode_parse(inst, argv[0]);
+	if (pc < 0 || !osmo_ss7_pc_is_valid((uint32_t)pc)) {
+		vty_out(vty, "Invalid point code (%s)%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	inst->cfg.secondary_pc = pc;
+	return CMD_SUCCESS;
+}
+
 /* TODO: cs7 capability-pc */
 DEFUN_ATTR(cs7_permit_dyn_rkm, cs7_permit_dyn_rkm_cmd,
 	   "xua rkm routing-key-allocation (static-only|dynamic-permitted)",
@@ -325,7 +360,7 @@ DEFUN_ATTR(cs7_route_table, cs7_route_table_cmd,
 }
 
 DEFUN_ATTR(cs7_rt_upd, cs7_rt_upd_cmd,
-	   "update route POINT_CODE MASK linkset LS_NAME [priority] [PRIO] [qos-class] [(CLASS|default)]",
+	   "update route POINT_CODE MASK linkset LS_NAME [priority] [" ROUTE_PRIO_VAR_STR "] [qos-class] [" QOS_CLASS_VAR_STR "]",
 	   "Update the Route\n"
 	   "Update the Route\n"
 	   "Destination Point Code\n"
@@ -333,10 +368,9 @@ DEFUN_ATTR(cs7_rt_upd, cs7_rt_upd_cmd,
 	   "Specify Destination Linkset\n"
 	   "Linkset Name\n"
 	   "Specify Priority\n"
-	   "Priority\n"
+	   ROUTE_PRIO_VAR_HELP_STR
 	   "Specify QoS Class\n"
-	   "QoS Class\n"
-	   "Default QoS Class\n",
+	   QOS_CLASS_VAR_HELP_STR,
 	   CMD_ATTR_IMMEDIATE)
 {
 	struct osmo_ss7_route_table *rtable = vty->index;
@@ -345,6 +379,7 @@ DEFUN_ATTR(cs7_rt_upd, cs7_rt_upd_cmd,
 	int mask = osmo_ss7_pointcode_parse_mask_or_len(rtable->inst, argv[1]);
 	const char *ls_name = argv[2];
 	unsigned int argind;
+	int rc;
 
 	if (dpc < 0) {
 		vty_out(vty, "%% Invalid point code (%s)%s", argv[0], VTY_NEWLINE);
@@ -356,16 +391,9 @@ DEFUN_ATTR(cs7_rt_upd, cs7_rt_upd_cmd,
 		return CMD_WARNING;
 	}
 
-	rt = osmo_ss7_route_create(rtable, dpc, mask, ls_name);
-	if (!rt) {
-		vty_out(vty, "%% Cannot create route %s/%s to %s%s",
-			argv[0], argv[1], argv[2], VTY_NEWLINE);
-		return CMD_WARNING;
-	}
-
 	switch (argc) {
 	case 3:
-		return CMD_SUCCESS;
+		break; /* Continue below */
 	case 5:
 		if (strcmp(argv[3], "priority") != 0 &&
 		    strcmp(argv[3], "qos-class") != 0)
@@ -381,18 +409,46 @@ DEFUN_ATTR(cs7_rt_upd, cs7_rt_upd_cmd,
 		return CMD_WARNING;
 	}
 
+	rt = ss7_route_alloc(rtable, dpc, mask);
+	if (!rt) {
+		vty_out(vty, "%% Cannot allocate new route%s", VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	if ((rc = ss7_route_set_linkset(rt, ls_name)) < 0) {
+		vty_out(vty, "%% Cannot find linkset %s%s", ls_name, VTY_NEWLINE);
+		goto destroy_warning;
+	}
+
 	argind = 3;
 	if (argc > argind && !strcmp(argv[argind], "priority")) {
 		argind++;
-		rt->cfg.priority = atoi(argv[argind++]);
+		if (strcmp(argv[argind], "default") != 0)
+			rt->cfg.priority = atoi(argv[argind]);
+		argind++;
 	}
 
 	if (argc > argind && !strcmp(argv[argind], "qos-class")) {
 		argind++;
-		rt->cfg.qos_class = atoi(argv[argind++]);
+		if (strcmp(argv[argind], "default") != 0)
+			rt->cfg.qos_class = atoi(argv[argind]);
+		argind++;
+	}
+
+	if ((rc = ss7_route_insert(rt)) < 0) {
+		char buf_err[128];
+		strerror_r(-rc, buf_err, sizeof(buf_err));
+		vty_out(vty, "%% Cannot insert route %s/%s to %s: %s (%d)%s",
+			argv[0], argv[1], argv[2],
+			buf_err, rc, VTY_NEWLINE);
+		goto destroy_warning;
 	}
 
 	return CMD_SUCCESS;
+
+destroy_warning:
+	ss7_route_destroy(rt);
+	return CMD_WARNING;
 }
 
 DEFUN_ATTR(cs7_rt_rem, cs7_rt_rem_cmd,
@@ -419,13 +475,13 @@ DEFUN_ATTR(cs7_rt_rem, cs7_rt_rem_cmd,
 		return CMD_WARNING;
 	}
 
-	rt = osmo_ss7_route_find_dpc_mask(rtable, dpc, mask);
+	rt = ss7_route_table_find_route_by_dpc_mask(rtable, dpc, mask);
 	if (!rt) {
 		vty_out(vty, "cannot find route to be deleted%s", VTY_NEWLINE);
 		return CMD_WARNING;
 	}
 
-	osmo_ss7_route_destroy(rt);
+	ss7_route_destroy(rt);
 	return CMD_SUCCESS;
 }
 
@@ -441,7 +497,7 @@ static void write_one_rtable(struct vty *vty, struct osmo_ss7_route_table *rtabl
 			osmo_ss7_pointcode_print(rtable->inst, rt->cfg.pc),
 			osmo_ss7_pointcode_print2(rtable->inst, rt->cfg.mask),
 			rt->cfg.linkset_name);
-		if (rt->cfg.priority)
+		if (rt->cfg.priority != OSMO_SS7_ROUTE_PRIO_DEFAULT)
 			vty_out(vty, " priority %u", rt->cfg.priority);
 		if (rt->cfg.qos_class)
 			vty_out(vty, " qos-class %u", rt->cfg.qos_class);
@@ -449,7 +505,8 @@ static void write_one_rtable(struct vty *vty, struct osmo_ss7_route_table *rtabl
 	}
 }
 
-static void vty_dump_rtable(struct vty *vty, struct osmo_ss7_route_table *rtbl)
+/* "filter_pc == OSMO_SS7_PC_INVALID" means "show all" */
+static void vty_dump_rtable(struct vty *vty, struct osmo_ss7_route_table *rtbl, uint32_t filter_pc)
 {
 	struct osmo_ss7_route *rt;
 
@@ -460,18 +517,34 @@ static void vty_dump_rtable(struct vty *vty, struct osmo_ss7_route_table *rtbl)
 	vty_out(vty, "---------------------- - - - ------------------- ------- ------- -------%s", VTY_NEWLINE);
 
 	llist_for_each_entry(rt, &rtbl->routes, list) {
-		vty_out(vty, "%-22s %c %c %u %-19s %-7s %-7s %-7s%s",
+		if ((filter_pc != OSMO_SS7_PC_INVALID) && ((filter_pc & rt->cfg.mask) != rt->cfg.pc))
+			continue; /* Skip routes not matching destination */
+
+		bool rt_avail = ss7_route_is_available(rt);
+
+		vty_out(vty, "%-16s %-5s %c %c %u %-19s %-7s %-7s %-7s%s",
 			osmo_ss7_route_print(rt),
-			' ', ' ', rt->cfg.priority, rt->cfg.linkset_name, "?", "?", "?", VTY_NEWLINE);
+			rt_avail ? "acces" : "INACC",
+			' ',
+			'0' + rt->cfg.qos_class,
+			rt->cfg.priority,
+			rt->cfg.linkset_name,
+			rt_avail ? "avail" : "UNAVAIL",
+			"?",
+			rt_avail ? "avail" : "UNAVAIL",
+			VTY_NEWLINE);
 	}
 }
 
 DEFUN(show_cs7_route, show_cs7_route_cmd,
-	"show cs7 instance <0-15> route",
-	SHOW_STR CS7_STR INST_STR INST_STR "Routing Table\n")
+	"show cs7 instance <0-15> route [POINT_CODE]",
+	SHOW_STR CS7_STR INST_STR INST_STR
+	"Routing Table\n"
+	"Destination Point Code\n")
 {
 	int id = atoi(argv[0]);
 	struct osmo_ss7_instance *inst;
+	uint32_t filter_pc = OSMO_SS7_PC_INVALID;
 
 	inst = osmo_ss7_instance_find(id);
 	if (!inst) {
@@ -479,7 +552,61 @@ DEFUN(show_cs7_route, show_cs7_route_cmd,
 		return CMD_WARNING;
 	}
 
-	vty_dump_rtable(vty, inst->rtable_system);
+	if (argc > 1) {
+		int pc = osmo_ss7_pointcode_parse(inst, argv[1]);
+		if (pc < 0 || !osmo_ss7_pc_is_valid((uint32_t)pc)) {
+			vty_out(vty, "Invalid point code (%s)%s", argv[1], VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		filter_pc = (uint32_t)pc;
+	}
+
+	vty_dump_rtable(vty, inst->rtable_system, filter_pc);
+	return CMD_SUCCESS;
+}
+
+DEFUN(show_cs7_route_lookup, show_cs7_route_lookup_cmd,
+      "show cs7 instance <0-15> route-lookup POINT_CODE from POINT_CODE sls <0-15> [list-asps]",
+      SHOW_STR CS7_STR INST_STR INST_STR
+      "Look up route\n" "Destination PC\n"
+      "From\n" "Origin PC\n"
+      "SLS\n" "SLS value\n"
+      "List ASPs of the AS if route points to an AS")
+{
+	int id = atoi(argv[0]);
+	bool list_asps = argc > 4;
+	struct osmo_ss7_instance *inst;
+	struct osmo_ss7_route *rt;
+	uint32_t dpc;
+	int pc;
+
+	inst = osmo_ss7_instance_find(id);
+	if (!inst) {
+		vty_out(vty, "No SS7 instance %d found%s", id, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	pc = osmo_ss7_pointcode_parse(inst, argv[1]);
+	if (pc < 0 || !osmo_ss7_pc_is_valid((uint32_t)pc)) {
+		vty_out(vty, "Invalid point code (%s)%s", argv[1], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+	dpc = pc;
+
+	pc = osmo_ss7_pointcode_parse(inst, argv[2]);
+	if (pc < 0 || !osmo_ss7_pc_is_valid((uint32_t)pc)) {
+		vty_out(vty, "Invalid point code (%s)%s", argv[2], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	rt = osmo_ss7_route_lookup(inst, dpc);
+	if (!rt) {
+		vty_out(vty, "No route found for DPC %s%s",
+			osmo_ss7_pointcode_print(inst, dpc), VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	vty_out(vty, "%s%s", osmo_ss7_route_name(rt, list_asps), VTY_NEWLINE);
 	return CMD_SUCCESS;
 }
 
@@ -530,13 +657,13 @@ DEFUN_ATTR(cs7_xua, cs7_xua_cmd,
 	if (trans_proto < 0)
 		return CMD_WARNING;
 
-	xs = osmo_ss7_xua_server_find2(inst, trans_proto, proto, port);
+	xs = ss7_xua_server_find2(inst, trans_proto, proto, port);
 	if (!xs) {
-		xs = osmo_ss7_xua_server_create2(inst, trans_proto, proto, port, NULL);
+		xs = ss7_xua_server_create2(inst, trans_proto, proto, port, NULL);
 		if (!xs)
 			return CMD_WARNING;
 		/* Drop first dummy address created automatically by _create(): */
-		osmo_ss7_xua_server_set_local_hosts(xs, NULL, 0);
+		ss7_xua_server_set_local_hosts(xs, NULL, 0);
 	}
 
 	vty->node = L_CS7_XUA_NODE;
@@ -565,12 +692,12 @@ DEFUN_ATTR(no_cs7_xua, no_cs7_xua_cmd,
 	if (trans_proto < 0)
 		return CMD_WARNING;
 
-	xs = osmo_ss7_xua_server_find2(inst, trans_proto, proto, port);
+	xs = ss7_xua_server_find2(inst, trans_proto, proto, port);
 	if (!xs) {
 		vty_out(vty, "No xUA server for port %u found%s", port, VTY_NEWLINE);
 		return CMD_WARNING;
 	}
-	osmo_ss7_xua_server_destroy(xs);
+	ss7_xua_server_destroy(xs);
 	return CMD_SUCCESS;
 }
 
@@ -583,7 +710,7 @@ DEFUN_ATTR(xua_local_ip, xua_local_ip_cmd,
 {
 	struct osmo_xua_server *xs = vty->index;
 
-	osmo_ss7_xua_server_add_local_host(xs, argv[0]);
+	ss7_xua_server_add_local_host(xs, argv[0]);
 
 	return CMD_SUCCESS;
 }
@@ -597,7 +724,7 @@ DEFUN_ATTR(xua_no_local_ip, xua_no_local_ip_cmd,
 {
 	struct osmo_xua_server *xs = vty->index;
 
-	if (osmo_ss7_xua_server_del_local_host(xs, argv[0]) != 0) {
+	if (ss7_xua_server_del_local_host(xs, argv[0]) != 0) {
 		vty_out(vty, "%% Failed deleting local address '%s' from set%s", argv[0], VTY_NEWLINE);
 		return CMD_WARNING;
 	}
@@ -696,7 +823,7 @@ static void vty_dump_xua_server(struct vty *vty, struct osmo_xua_server *xs)
 	int fd = xs->server ? osmo_stream_srv_link_get_fd(xs->server) : -1;
 
 	if (fd < 0) {
-		if (osmo_ss7_asp_peer_snprintf(buf, sizeof(buf), &xs->cfg.local) < 0)
+		if (ss7_asp_peer_snprintf(buf, sizeof(buf), &xs->cfg.local) < 0)
 			snprintf(buf, sizeof(buf), "<error>");
 	} else {
 		char hostbuf[OSMO_SOCK_MAX_ADDRS][INET6_ADDRSTRLEN];
@@ -910,7 +1037,7 @@ DEFUN_ATTR(asp_local_ip, asp_local_ip_cmd,
 	int old_host_count = asp->cfg.local.host_cnt;
 	int rc;
 
-	if (osmo_ss7_asp_peer_add_host2(&asp->cfg.local, asp, argv[0], is_primary) != 0) {
+	if (ss7_asp_peer_add_host2(&asp->cfg.local, asp, argv[0], is_primary) != 0) {
 		vty_out(vty, "%% Failed adding host '%s' to set%s", argv[0], VTY_NEWLINE);
 		return CMD_WARNING;
 	}
@@ -972,7 +1099,7 @@ DEFUN_ATTR(asp_no_local_ip, asp_no_local_ip_cmd,
 		}
 	}
 
-	if (osmo_ss7_asp_peer_del_host(&asp->cfg.local, argv[0]) != 0) {
+	if (ss7_asp_peer_del_host(&asp->cfg.local, argv[0]) != 0) {
 		vty_out(vty, "%% Failed deleting local address '%s' from set%s", argv[0], VTY_NEWLINE);
 		return CMD_WARNING;
 	}
@@ -992,7 +1119,7 @@ DEFUN_ATTR(asp_remote_ip, asp_remote_ip_cmd,
 	int old_idx_primary = asp->cfg.remote.idx_primary;
 	int rc;
 
-	if (osmo_ss7_asp_peer_add_host2(&asp->cfg.remote, asp, argv[0], is_primary) != 0) {
+	if (ss7_asp_peer_add_host2(&asp->cfg.remote, asp, argv[0], is_primary) != 0) {
 		vty_out(vty, "%% Failed adding host '%s' to set%s", argv[0], VTY_NEWLINE);
 		return CMD_WARNING;
 	}
@@ -1029,7 +1156,7 @@ DEFUN_ATTR(asp_no_remote_ip, asp_no_remote_ip_cmd,
 		return CMD_WARNING;
 	}
 
-	if (osmo_ss7_asp_peer_del_host(&asp->cfg.remote, argv[0]) != 0) {
+	if (ss7_asp_peer_del_host(&asp->cfg.remote, argv[0]) != 0) {
 		vty_out(vty, "%% Failed deleting remote address '%s' from set%s", argv[0], VTY_NEWLINE);
 		return CMD_WARNING;
 	}
@@ -1037,9 +1164,9 @@ DEFUN_ATTR(asp_no_remote_ip, asp_no_remote_ip_cmd,
 }
 
 DEFUN_ATTR(asp_qos_clas, asp_qos_class_cmd,
-	   "qos-class <0-255>",
+	   "qos-class " QOS_CLASS_RANGE_STR,
 	   "Specify QoS Class of ASP\n"
-	   "QoS Class of ASP\n",
+	   QOS_CLASS_RANGE_HELP_STR,
 	   CMD_ATTR_NODE_EXIT)
 {
 	struct osmo_ss7_asp *asp = vty->index;
@@ -1335,8 +1462,8 @@ static void show_one_asp(struct vty *vty, struct osmo_ss7_asp *asp)
 		if (!get_sockname_buf(buf_rem, sizeof(buf_rem), fd, trans_proto, false))
 			OSMO_STRLCPY_ARRAY(buf_rem, "<sockname-error>");
 	} else {
-		osmo_ss7_asp_peer_snprintf(buf_loc, sizeof(buf_loc), &asp->cfg.local);
-		osmo_ss7_asp_peer_snprintf(buf_rem, sizeof(buf_rem), &asp->cfg.remote);
+		ss7_asp_peer_snprintf(buf_loc, sizeof(buf_loc), &asp->cfg.local);
+		ss7_asp_peer_snprintf(buf_rem, sizeof(buf_rem), &asp->cfg.remote);
 	}
 
 	vty_out(vty, "%-12s  %-12s  %-13s  %-4s  %-4s  %-9s  %-23s  %-23s%s",
@@ -1898,9 +2025,9 @@ DEFUN_ATTR(as_recov_tout, as_recov_tout_cmd,
 }
 
 DEFUN_ATTR(as_qos_clas, as_qos_class_cmd,
-	   "qos-class <0-255>",
+	   "qos-class " QOS_CLASS_RANGE_STR,
 	   "Specity QoS Class of AS\n"
-	   "QoS Class of AS\n",
+	   QOS_CLASS_RANGE_HELP_STR,
 	   CMD_ATTR_IMMEDIATE)
 {
 	struct osmo_ss7_as *as = vty->index;
@@ -1971,9 +2098,9 @@ static int _rout_key(struct vty *vty,
 	 * (users may change the routing key via VTY during runtime) and then
 	 * putting a new route (see below). */
 	if (cs7_role == CS7_ROLE_ASP) {
-		rt = osmo_ss7_route_find_dpc_mask(as->inst->rtable_system, rkey->pc, 0xffffff);
+		rt = ss7_route_table_find_route_by_dpc_mask(as->inst->rtable_system, rkey->pc, 0xffffff);
 		if (rt)
-			osmo_ss7_route_destroy(rt);
+			ss7_route_destroy(rt);
 	}
 
 	rkey->pc = pc;
@@ -1984,7 +2111,7 @@ static int _rout_key(struct vty *vty,
 
 	/* automatically add new route (see also comment above) */
 	if (cs7_role == CS7_ROLE_ASP) {
-		if (!osmo_ss7_route_create(as->inst->rtable_system, rkey->pc, 0xffffff, as->cfg.name)) {
+		if (!ss7_route_create(as->inst->rtable_system, rkey->pc, 0xffffff, as->cfg.name)) {
 			vty_out(vty, "Cannot create route (pc=%s, linkset=%s) to AS %s", dpc, as->cfg.name, VTY_NEWLINE);
 			return CMD_WARNING;
 		}
@@ -2829,6 +2956,11 @@ static void write_one_cs7(struct vty *vty, struct osmo_ss7_instance *inst, bool
 			osmo_ss7_pointcode_print(inst, inst->cfg.primary_pc),
 			VTY_NEWLINE);
 
+	if (osmo_ss7_pc_is_valid(inst->cfg.secondary_pc)) {
+		vty_out(vty, " secondary-pc %s%s",
+			osmo_ss7_pointcode_print(inst, inst->cfg.secondary_pc), VTY_NEWLINE);
+	}
+
 	if (inst->cfg.permit_dyn_rkm_alloc)
 		vty_out(vty, " xua rkm routing-key-allocation dynamic-permitted%s", VTY_NEWLINE);
 
@@ -2869,9 +3001,7 @@ int osmo_ss7_vty_go_parent(struct vty *vty)
 	switch (vty->node) {
 	case L_CS7_ASP_NODE:
 		asp = vty->index;
-		/* Make sure proper defaults values are set */
-		ss7_asp_set_default_peer_hosts(asp);
-		osmo_ss7_asp_restart(asp);
+		ss7_asp_restart_after_reconfigure(asp);
 		vty->node = L_CS7_NODE;
 		vty->index = asp->inst;
 		break;
@@ -2889,7 +3019,7 @@ int osmo_ss7_vty_go_parent(struct vty *vty)
 		oxs = vty->index;
 		/* If no local addr was set, or erased after _create(): */
 		ss7_xua_server_set_default_local_hosts(oxs);
-		if (osmo_ss7_xua_server_bind(oxs) < 0)
+		if (ss7_xua_server_bind(oxs) < 0)
 			vty_out(vty, "%% Unable to bind xUA server to IP(s)%s", VTY_NEWLINE);
 		vty->node = L_CS7_NODE;
 		vty->index = oxs->inst;
@@ -2912,6 +3042,8 @@ int osmo_ss7_vty_go_parent(struct vty *vty)
 	return 0;
 }
 
+/* This is no longer used. The libosmocore callback was deprecated is not ever called since.
+ * libosmocore.git 70ce871532ab21955e0955d7e230eae65438f047 (release 1.3.0). */
 int osmo_ss7_is_config_node(struct vty *vty, int node)
 {
 	switch (node) {
@@ -2989,6 +3121,7 @@ static void vty_init_shared(void *ctx)
 	install_lib_element(L_CS7_NODE, &cfg_description_cmd);
 	install_lib_element(L_CS7_NODE, &cs7_net_ind_cmd);
 	install_lib_element(L_CS7_NODE, &cs7_point_code_cmd);
+	install_lib_element(L_CS7_NODE, &cs7_secondary_pc_cmd);
 	install_lib_element(L_CS7_NODE, &cs7_pc_format_cmd);
 	install_lib_element(L_CS7_NODE, &cs7_pc_format_def_cmd);
 	install_lib_element(L_CS7_NODE, &cs7_pc_delimiter_cmd);
@@ -3041,6 +3174,7 @@ static void vty_init_shared(void *ctx)
 	install_lib_element(L_CS7_AS_NODE, &as_pc_patch_sccp_cmd);
 
 	install_lib_element_ve(&show_cs7_route_cmd);
+	install_lib_element_ve(&show_cs7_route_lookup_cmd);
 
 	vty_init_addr();
 }
